/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 19/09/2001
**/
#include <qpainter.h>
#include <ctype.h>
#include "cono.h"

Cono::Cono( QWidget *parent )
		: QGLWidget( parent )
//  : QGLWidget( QGLFormat( QGL::SingleBuffer ), parent )
//  : QGLWidget( QGLFormat( QGL::SingleBuffer|QGL::NoDepthBuffer ), parent )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	// setFocusPolicy( QWidget::StrongFocus);
	setFocusPolicy( Qt::StrongFocus);

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;

	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	n = 10;
	creaCono( c1, 8.0, 5.0 );
}


void Cono::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Cono::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Cono::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Cono::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	// En z estn en coordenadas negativas
	if( w <= h )
		glOrtho ( -10.0, 10.0, -10.0/aspect, 10.0/aspect, -10.0, 10.0);
	else
		glOrtho (-10.0*aspect, 10.0*aspect, -10.0, 10.0, -10.0, 10.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	emit changeSize ( );
}

QSizePolicy Cono::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Cono::creaCono( float v[][3], float l, float r )
{
	int i;
	float theta, ang;

	theta = 2.0*3.141592654/n;

	for( i=0; i<n; i++ ) {
		ang = theta*i;
		v[i][0] = r*cos( ang );
		v[i][1] = 0.0;
		v[i][2] = r*sin( ang );
	}
	v[n][0] = 0; v[n][1] = l; v[n][2] = 0;
}

void Cono::dibujaCono( float v[][3] )
{
	int i, k;

	for( i=0; i<n; i++ ){
		k = (i+1)%n;

		glBegin( GL_TRIANGLES );
		glColor3f( 1.0, 1.0, 1.0 );
		glVertex3fv( &v[ i ][0] );
		glVertex3fv( &v[ k ][0] );

		glColor3f( 1.0, 0.0, 0.0 );
		glVertex3fv( &v[ n ][0] );
		glEnd( );
	}
}

void Cono::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix( );
	glRotatef (yrot, 0.0, 1.0, 0.0);
	glRotatef (xrot, 1.0, 0.0, 0.0);

	dibujaCono( c1 );

	glPopMatrix( );

	swapBuffers( );
}

void Cono::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

	glEnable (GL_DEPTH_TEST);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */
}


